#include "lsgrinter.hpp"

bool
Lsgrinter::draw()
{
	header();
	
	for(actionType::iterator it = result_.begin(); it != result_.end(); ++it) {

		//metasymbols
		switch(*it) {
			case meta_value_push:
			case meta_value_pop:
			case meta_value_xp:
			case meta_value_xn:
			case meta_value_yp:
			case meta_value_yn:
			case meta_value_zp:
			case meta_value_zn:
				if(!meta(*it))
					return false;
				break;
			default:
				nonmeta(*it);
				break;
		}
	}

	footer();

	return true;
}

void
Lsgrinter::header()
{
	//xml header
	fout << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
		<< "<X3D profile='Immersive'>\n"
		<< "<Scene>\n";
}

void
Lsgrinter::footer()
{
	//footer
	fout << "</Scene>\n"
		<< "</X3D>\n";

}

bool 
Lsgrinter::meta(int sym) 
{
	double foo;

	switch(sym) {
		case meta_value_push:
			states.push(state(x_, y_, z_, rx_, ry_, rz_, ax_, ay_, az_));
			break;
		case meta_value_pop:
			if(states.empty()) {
				error = "Error: Renew state where no state on stack";
				return false;
			}

			x_ = states.top().x_;
			y_ = states.top().y_;
			z_ = states.top().z_;
			rx_ = states.top().rx_;
			ry_ = states.top().ry_;
			rz_ = states.top().rz_;
			ax_ = states.top().ax_;
			ay_ = states.top().ay_;
			az_ = states.top().az_;

			states.pop();

			break;
		case meta_value_xp:
			ax_ += angle_;
			foo = ry_;
			ry_ = ry_ * cos(angle_) - rz_ * sin(angle_);
			rz_ = foo * sin(angle_) + rz_ * cos(angle_);
			break;
		case meta_value_xn:
			ax_ -= angle_;
			foo = ry_;
			ry_ = ry_ * cos(-angle_) - rz_ * sin(-angle_);
			rz_ = foo * sin(-angle_) + rz_ * cos(-angle_);
			break;
		case meta_value_yp:
			ay_ += angle_;
			foo = rz_;
			rz_ = rz_ * cos(angle_) - rx_ * sin(angle_);
			rx_ = foo * sin(angle_) + rx_ * cos(angle_);
			break;
		case meta_value_yn:
			ay_ -= angle_;
			foo = rz_;
			rz_ = rz_ * cos(-angle_) - rx_ * sin(-angle_);
			rx_ = foo * sin(-angle_) + rx_ * cos(-angle_);
			break;
		case meta_value_zp:
			az_ += angle_;
			foo = rx_;
			rx_ = rx_ * cos(angle_) - ry_ * sin(angle_);
			ry_ = foo * sin(angle_) + ry_ * cos(angle_);
			break;
		case meta_value_zn:
			az_ -= angle_;
			foo = rx_;
			rx_ = rx_ * cos(-angle_) - ry_ * sin(-angle_);
			ry_ = foo * sin(-angle_) + ry_ * cos(-angle_);
			break;
	}

	return true;
}

void 
Lsgrinter::nonmeta(int sym) 
{
	double nx, ny, nz;

	nx = (x_ + x_ + rx_) / 2;
	ny = (y_ + y_ + ry_) / 2;
	nz = (z_ + z_ + rz_) / 2;
	
	x_ += rx_; y_ += ry_; z_ += rz_;

	//if symbol is to be drawn
	if(std::find(forwards_.begin(), forwards_.end(), sym) != forwards_.end()) {
		fout	<< "<Transform translation='" << nx  << " " << ny << " " << nz << "'>\n"
			<< "\t<Transform " << "rotation='" << 1 << " " << 0 << " " << 0 << " " << ax_ << "'>\n"
			<< "\t\t<Transform " << "rotation='" << 0 << " " << 1 << " " << 0 << " " << ay_ << "'>\n"
			<< "\t\t\t<Transform " << "rotation='" << 0 << " " << 0 << " " << 1 << " " << az_ << "'>\n"
			<< "\t\t\t\t<Shape>\n"
			<< "\t\t\t\t\t<Cylinder height='" << step << "' radius='" << radius << "' />\n"
			<< "\t\t\t\t</Shape>\n"
			<< "\t\t\t</Transform>\n"
			<< "\t\t</Transform>\n"
			<< "\t</Transform>\n"
			<< "</Transform>\n";
	}
}
