#include "lsgen.hpp"

#include <string>
#include <fstream>
#include <iostream>
#include <algorithm>

#include <stack>

#include <cmath>
#include <cstdlib>

#ifndef LSGRINTER
#define LSGRINTER

#define step 1
#define radius 0.1

class Lsgrinter {
private:
	struct state {
		double x_, y_, z_;
		double rx_, ry_, rz_;
		double ax_, ay_, az_;

		state(double x, double y, double z,
			double rx, double ry, double rz,
			double ax, double ay, double az) :
			x_(x), y_(y), z_(z),
			rx_(rx), ry_(ry), rz_(rz),
			ax_(ax), ay_(ay), az_(az) {}
	};

	actionType result_;
	forwardsType forwards_;

	const char* file_;

	std::fstream fout;

	//position in universe
	double x_, y_, z_;

	//current move vector
	double rx_, ry_, rz_;

	//current orientation
	double ax_, ay_, az_;

	double angle_;

	std::stack< state> states;

public:
	Lsgrinter(const char* file, actionType result, forwardsType forwards, double angle) : 
		file_(file),
		result_(result),
		forwards_(forwards),
		angle_(angle) {

		fout.open(file_, std::ios_base::out);
		fout.precision(6);
		fout.setf(std::ios::fixed,std::ios::floatfield);

		rx_ = rz_ = 0; ry_ = step;
		ax_ = ay_ = az_ = 0;
		x_ = y_ = z_ = 0;
	}

	~Lsgrinter() {
		fout.close();
	}
	
	bool draw();

	void header();
	bool meta(int);
	void nonmeta(int);
	void footer();

	std::string error;
};

#endif
