#include "lsparser.hpp"

bool
Lsparser::parse()
{
	std::ifstream fin(path_.c_str(), std::ios_base::in);

	if (!fin) {
	        error = "Error: Could not open input file: " + path_;
		return false;
	}

	std::string input; //file content
	std::copy(
		std::istream_iterator<char>(fin),
		std::istream_iterator<char>(),
		std::back_inserter(input));


	std::string::iterator foo, bar;
	foo = input.begin();
	bar = input.end();


	bool r = boost::spirit::qi::phrase_parse(
		foo,
		bar,
		(KEYWORD_AXIOM >> elements[boost::phoenix::ref(axiom) = boost::spirit::qi::_1]) ^
		(KEYWORD_ANGLE >> boost::spirit::qi::double_[boost::phoenix::ref(angle) = boost::spirit::qi::_1]) ^
		(KEYWORD_GENERATIONS >> boost::spirit::qi::int_[boost::phoenix::ref(gens) = boost::spirit::qi::_1]) ^
		(KEYWORD_FORWARDS >> *(elements[boost::phoenix::push_back(boost::phoenix::ref(forwards), boost::spirit::qi::_1)])) ^
		(KEYWORD_RULES >> 
			*(elements[boost::bind(&Lsparser::precondition, this, _1)] >> KEYWORD_PRODUCTS >> 
			+elements[boost::bind(&Lsparser::action, this, _1)] >> 
			boost::spirit::qi::lit(';')[boost::bind(&Lsparser::store, this)])) >>
		boost::spirit::qi::eoi,
		boost::spirit::ascii::space);

	return r;
}

void
Lsparser::precondition(int prec)
{
	prec_ = prec;
}

void
Lsparser::action(int act)
{
	act_.push_back(act);
}

void 
Lsparser::store()
{
	rules.insert(std::pair<int, actionType>(prec_, act_));
	act_.clear();
}
