#include <string>
#include <list>
#include <map>
#include <fstream>

#include <boost/spirit/include/qi.hpp>
#include <boost/spirit/include/phoenix_core.hpp>
#include <boost/spirit/include/phoenix_operator.hpp>
#include <boost/spirit/include/phoenix_stl.hpp>
#include <boost/bind.hpp>

#include "lsgen.hpp"

#ifndef LSPARSER
#define LSPARSER

class Lsparser {
	
	boost::spirit::qi::rule< std::string::iterator, unsigned()> start;

	std::string path_;

	//alloeb elements
	struct elements_ : boost::spirit::qi::symbols<char, int> {
		elements_() {
			add
				//production elements
				(symbol_a, value_a)
				(symbol_b, value_b)
				(symbol_c, value_c)
				(symbol_d, value_d)
				(symbol_e, value_e)
				(symbol_f, value_f)
				(symbol_g, value_g)
				(symbol_h, value_h)

				//meta elements
				(meta_symbol_push, meta_value_push)
				(meta_symbol_pop, meta_value_pop)
				(meta_symbol_xp, meta_value_xp)
				(meta_symbol_xn, meta_value_xn)
				(meta_symbol_yp, meta_value_yp)
				(meta_symbol_yn, meta_value_yn)
				(meta_symbol_zp, meta_value_zp)
				(meta_symbol_zn, meta_value_zn)
			;}
	} elements;


	int prec_;
	actionType act_;

	void precondition(int); //production rule precondition
	void action(int); //production rule action - one of many
	void store(); //store production rule

public:

	int axiom; //initial axiom of L-system
	double angle; //rotation angle
	int gens; //number of generations

	forwardsType forwards; //forward elements
	rulesType rules; //production rules

	//error message
	std::string error;
	
	//ctor
	Lsparser(std::string path) : path_(path) {};

	//parse input file and store results
	bool
	parse();
};

#endif
